<?php
class Api {

	public	$has_error		= false,
					$socket 			=	null,
					$socket_error	= '';

	private	$_host = '127.0.0.1',
					$_port = '55542';

	/* [CONSTRUCTOR] */
	function __construct() {
		// check sbc-api running
		if ($this->_isBeronodeApiRunning()) {
			return;
		}
		$this->has_error = true;
		$this->socket_error = "[code=500|msg=beronode not running]";
	}

	/* [PUBLIC METHODS] */
	public function exec(string $cmd) {
		if ($this->has_error) {
			return $this->getError();
		}
		return $this->sendRequest('post', "/ExecuteCommand", ['cmd' => $cmd]);
	}

	public function getError() {
		return ['error' => $this->socket_error];
	}

	public function sendRequest($method, $uri, $data = null, $options = []) {
		// TODO: write a generic function in the Helper class
		$api_url = "http://{$this->_host}:{$this->_port}$uri";
	
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $api_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);

		switch ($method) {
		case 'delete':
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
			break;
		case 'get':
			break;
		case 'post':
			curl_setopt($ch, CURLOPT_POST, 1);
			break;
		case 'put':
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
			break;
		default:
			curl_close($ch);
			return false;
		}

		if (!is_null($data)) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
		}
		curl_setopt($ch, CURLOPT_TIMEOUT, isset($options['timeout']) ? (int) $options['timeout'] : 1);

		$response = curl_exec($ch);
		curl_close($ch);
		return(isset($options['withoutJsonDecode']) && $options['withoutJsonDecode'] ? $response : json_decode($response, true));
	}

	/* [PRIVATE METHODS] */
	private function _isBeronodeApiRunning() {
		try {
			$this->socket = @fsockopen($this->_host, $this->_port, $errno, $errstr, 1);
		}
		catch (Exception $e) {
			$this->socket_error = "[code={$e->getCode()}|msg={$e->getMessage()}|file={$e->getFile()}|line={$e->getLine()}]";
			return false;
		}
		return $this->socket;
	}
}
?>
